/*
 *  Proxy-applet.cs: a tray applet for direct access to proxy preferences.
 *  Copyright 2003 Dag Wieers <dag@wieers.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Diagnostics;

using Gnome;
using GnomeSharp;
using Gtk;
using GtkSharp;
using Glade;

using Egg;

public class ProxyTrayIconApplication {
	public static void Main() {
		Application.Init();
		TrayIcon tray=new ProxyTrayIcon("Proxy Applet");
		tray.ShowAll();
		Application.Run();
	}
}

public class ProxyTrayIcon : TrayIcon {
	const string NAME = "Proxy Applet";
	const string VERSION = "0.2.2";

	private EventBox eb = new EventBox();
	private GConf.Client gc1 = new GConf.Client();
	private GConf.Client gc2 = new GConf.Client();
	private GConf.Client gc3 = new GConf.Client();
	private GConf.Client gc4 = new GConf.Client();
	private Menu leftmenu, rightmenu;
	private RadioMenuItem set1, set2, set3;
	private Tooltips traytooltip = new Tooltips();
	private Glade.XML gxml;

	Gnome.About about;

	public ProxyTrayIcon (string name) : base(name) {

		/* Left-mousebutton menu */
		leftmenu = new Menu();
		
		ImageMenuItem it_pref  = new ImageMenuItem(Gtk.Stock.Preferences, new Gtk.AccelGroup());
		it_pref.Activated += new EventHandler(Preferences);
		leftmenu.Append(it_pref);
// Test buttons
/*		ImageMenuItem it_save  = new ImageMenuItem(Gtk.Stock.SaveAs, new Gtk.AccelGroup());
		it_save.Name = "Add profile";
		leftmenu.Append(it_save);
		ImageMenuItem it_open  = new ImageMenuItem(Gtk.Stock.Open, new Gtk.AccelGroup());
		leftmenu.Append(it_open);
*/
		leftmenu.Append(new SeparatorMenuItem());

		GLib.SList radiogroup = new GLib.SList(IntPtr.Zero);
		set1=new RadioMenuItem(radiogroup, "No proxy");
		radiogroup=set1.Group;
		set2=new RadioMenuItem(radiogroup, "Manual proxy");
		radiogroup=set2.Group;
		set3=new RadioMenuItem(radiogroup, "Auto proxy");

		checkProxyPref();
		getProxyPref(set1, null);

		set1.Activated += new EventHandler(setProxyPref);
		set2.Activated += new EventHandler(setProxyPref);
		set3.Activated += new EventHandler(setProxyPref);

		leftmenu.Append(set1);
		leftmenu.Append(set2);
		leftmenu.Append(set3);


		/* Right-mousebutton menu */
		rightmenu = new Menu();

//		FIXME: gnome-stock-about should work !
//		ImageMenuItem it_about  = new ImageMenuItem(Gnome.Stock.About, new Gtk.AccelGroup());
		ImageMenuItem it_about  = new ImageMenuItem("About...", new Gtk.AccelGroup());
//		Gtk.ItemFactoryEntry it_about = new Gtk.ItemFactoryEntry(Gnome.Stock.About);
		it_about.Activated += new EventHandler(About);
		rightmenu.Append(it_about);

		rightmenu.Append(new SeparatorMenuItem());

		ImageMenuItem it_remove  = new ImageMenuItem(Gtk.Stock.Remove, new Gtk.AccelGroup());
		it_remove.Name="Remove Icon";
		it_remove.Activated += new EventHandler(Quit);
		rightmenu.Append(it_remove);

		eb.ButtonPressEvent += new ButtonPressEventHandler(IconClicked);
		eb.Add (new Image(new Gdk.Pixbuf(null, "pixmaps/proxy-applet.png")));	// FIXME: resize icon according to panel

		this.Add(eb);
	}

	// Set default GConf proxy values, if they don't exist.
	private void checkProxyPref() {
		// FIXME: Do I really need to have a seperate GConf.Client for each EventHandler ?
		gc1.AddNotify("/system/proxy/mode", new GConf.NotifyEventHandler(getProxyPref));
		try {
			gc1.Get("/system/proxy/mode");
		} catch(GConf.NoSuchKeyException) {
			gc1.Set("/system/proxy/mode", "none");
		}
		gc2.AddNotify("/system/http_proxy/host", new GConf.NotifyEventHandler(getProxyPref));
		try {
			gc2.Get("/system/http_proxy/host");
		} catch(GConf.NoSuchKeyException) {
			gc2.Set("/system/http_proxy/host", "proxy");
		}
		gc3.AddNotify("/system/http_proxy/port", new GConf.NotifyEventHandler(getProxyPref));
		try {
			gc3.Get("/system/http_proxy/port");
		} catch(GConf.NoSuchKeyException) {
			gc3.Set("/system/http_proxy/port", "8080");
		}
		gc4.AddNotify("/system/proxy/autoconfig_url", new GConf.NotifyEventHandler(getProxyPref));
		try {
			gc4.Get("/system/proxy/autoconfig_url");
		} catch(GConf.NoSuchKeyException) {
			gc4.Set("/system/proxy/autoconfig_url", "http://pac:8080/");
		}
	}

	// Get GConf proxy settings and update visuals.
	private void getProxyPref(object o, GConf.NotifyEventArgs args) {
		switch((string) gc1.Get("/system/proxy/mode")) {
			case "none":
				set1.Active=true;
				traytooltip.SetTip(eb, "Direct internet connection\nNo proxy", "");
				break;
			case "manual":
				set2.Active=true;
				string host = (string) gc2.Get("/system/http_proxy/host");
				int port = (int) gc3.Get("/system/http_proxy/port");
				traytooltip.SetTip(eb, "Manual proxy configuration\nProxy: " + host + ":" + port, "");
				break;
			case "auto":
				set3.Active=true;
				string url = (string) gc4.Get("/system/proxy/autoconfig_url");
				traytooltip.SetTip(eb, "Automatic proxy configuration\nURL: " + url, "");
				break;
//			default:
			// FIXME: If none of the above, warn user !
		}
	}

	// Change GConf proxy values when requested.
	private void setProxyPref(object o, EventArgs args) {
		if (o==set1) 
			gc1.Set("/system/proxy/mode", "none");
		else if (o==set2)
			gc1.Set("/system/proxy/mode", "manual");
		else if (o==set3)
			gc1.Set("/system/proxy/mode", "auto");
	}

	private void IconClicked(object o, ButtonPressEventArgs args) {
		if (args.Event.button == 3) { 
			rightmenu.ShowAll();
			rightmenu.Popup(null, null, null, IntPtr.Zero, args.Event.button, args.Event.time);
		} else if (args.Event.button == 1) {
			leftmenu.ShowAll();
			leftmenu.Popup(null, null, null, IntPtr.Zero, args.Event.button, args.Event.time);
		}
	}

	private void Preferences(object o, EventArgs args) {
		Process.Start("gnome-network-preferences");
	}

	private void About(object o, EventArgs args) {
		if (about==null) {
			gxml=new Glade.XML(null, "proxy-applet.glade", "about", null);
			about = (Gnome.About) gxml["about"];
			about.Hide();	// Don't show until built
			about.Name = NAME;
			about.Version = VERSION;
//			FIXME: This doesn't work for my embedded resources inside a glade-file ;-(
			about.Logo = new Gdk.Pixbuf(null, "pixmaps/proxy-applet-48x48.png");
			about.Icon = new Gdk.Pixbuf(null, "pixmaps/proxy-applet.png");     
			about.Close += new EventHandler(OnAboutClose);
			about.Response += new ResponseHandler(OnAboutResponse);
			about.Show();
		} else {
			about.Deiconify();	// Bring up to front
		}
	}

	//FIXME: should be fixed upstream
	private void OnAboutClose(object o, EventArgs args) {
		about=null;
	}

	//FIXME: should be fixed upstream
	private void OnAboutResponse(object o, ResponseArgs args) {
		about=null;
	}

	private void Quit(object o, EventArgs args) {
		Application.Quit();
	}
}
